/*   $OSSEC, snort.c, v0.1, 2005/06/21, Daniel B. Cid$   */

/* Copyright (C) 2005 Daniel B. Cid <dcid@ossec.net>
 * All right reserved.
 *
 * This program is a free software; you can redistribute it
 * and/or modify it under the terms of the GNU General Public
 * License (version 2) as published by the FSF - Free Software
 * Foundation
 */

/* Snort decoder */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "analysisd/eventinfo.h"
#include "os_regex/os_regex.h"

/* Special decoder for snort
 * We are not using the default rendering
 * to make it simple (and less resource intensive)
 */
int DecodeSnort(Eventinfo *lf)
{
    char **ret;
    
    #ifdef DEBUG
    printf("%s: DEBUG: Checking snort decoder.\n",ARGV0);
    #endif
    
    /* Begging of the snort msg */
    if(!OS_Match("^[**] [",lf->log))
        return(0);


    /* Some examples
       [**] [1:1054:7] WEB-MISC weblogic/tomcat .jsp view source attempt [**] 
       [Classification: Web Application Attack] 
       [Priority: 1]  10.4.12.26:34041 -> 66.179.53.37:80

       [**] [1:1421:11] SNMP AgentX/tcp request [**] 
       [Classification: Attempted Information Leak] [Priority: 2]  
       10.4.3.20:626 -> 10.4.10.161:705

       [**] [1:1882:10] ATTACK-RESPONSES id check returned userid [**] 
       [Classification: Potentially Bad Traffic] [Priority: 2] 
       {UDP} 192.168.20.32 -> 192.168.20.2
     */

    /* getting snort ID */
    lf->type = SNORT;
   
    ret = OS_RegexStr("^[**] [(\\d+:\\d+:\\d+)]\\.+[Priority: \\d+]  "
             "(\\S+):\\d+ ->", lf->log);          


    /* If snort fast */
    if(ret == NULL)
    {
       ret = OS_RegexStr("^[**] [(\\d+:\\d+:\\d+)]\\.+[Priority: \\d+]  "
            "{\\S+} (\\S+):\\d+ ->", lf->log); 
    }

    
    /* Didn't match */
    if(ret == NULL)
        return(0);
    
    if(ret[0])
    {
        lf->id = strdup(ret[0]);
        free(ret[0]);
    }
    
    if(ret[1])
    {
        lf->srcip = strdup(ret[1]);
        free(ret[1]);
    }
    
    free(ret); 

    /* Snort FTS */
    lf->fts = strdup("name,id,srcip");
    lf->comment = strdup("First time Snort rule fired");
    
    return(1);
}
