#include <stdio.h>
#include <stdlib.h>
#include <string.h>


#include "headers/defs.h"

#include "rules.h"
#include "config.h"
#include "analysisd.h"
#include "eventinfo.h"

#include "headers/debug_op.h"

#include "os_regex/os_regex.h"

/* Search LastEvents.  
 * Will look if any of the last events (inside the timeframe)
 * match the specified rule. 
 */
Eventinfo *Search_LastEvents()
{
    EventNode *eventnode_pt;

    eventnode_pt = OS_GetLastEvent();

    if(!eventnode_pt)
    {
        /* Nothing found */
        return(NULL);
    }
    
    /* Searching all previous events */
    do
    {
        Eventinfo *lf = eventnode_pt->event;
        
        /* If time is outside the timeframe, return */
        if((c_time - lf->time) > currently_rule->timeframe)
            return(NULL);
        
        /* If regex does not match, go to next */
        if(currently_rule->if_matched_regex)
        {
            if(!OS_Regex(currently_rule->if_matched_regex,
                    lf->log))
            {
                /* Didn't match */
                continue;
            }
        }

        /* Group match */
        if(currently_rule->if_matched_group)
        {
            if(!OS_Match(currently_rule->if_matched_group,
                lf->log))
            {
                continue; /* Didn't match */
            }
        }
       
        /* checking for repetitions on user error */
        if(currently_rule->same_user)
        {
            if((!lf->user)||(!currently_lf->user))
                continue;
                
            if(strcmp(lf->user,currently_lf->user) != 0)
                continue;
            
            if(currently_lf->frequency < currently_rule->frequency)
            {
                currently_lf->frequency++;
                continue;
            }
        }
        
        /* checking for repetitions from same src_ip */
        if(currently_rule->same_source_ip)
        {
            if((!lf->srcip)||(!currently_lf->srcip))
                continue;
                
            if(strcmp(lf->srcip,currently_lf->srcip) != 0)
                continue;
            
            if(currently_lf->frequency < currently_rule->frequency)
            {
                currently_lf->frequency++;
                continue;
            }
        }
        
        /* If reached here, we matched */
        return(lf);    
        
    }while((eventnode_pt=eventnode_pt->next) != NULL);

    return(NULL);
}


/* Zero the loginfo structure */
void Zero_Eventinfo(Eventinfo *lf)
{
    lf->log_tag = NULL;
    lf->log = NULL;
    lf->location = NULL;
    lf->group = NULL;
    lf->hostname = NULL;
    lf->comment = NULL;
    lf->info = NULL;

    lf->srcip = NULL;
    lf->dstip = NULL;
    lf->user = NULL;
    lf->dstuser = NULL;
    lf->id = NULL;
    lf->command = NULL;
    lf->fts = NULL;

    lf->type = 0;        
    lf->level = -1; /* level 0 is valid */
    lf->sigid = -1; /* signature id 0 is valid */
    lf->time = 0;
    lf->frequency = 0;
    
    lf->year = 0;
    lf->mon = NULL;
    lf->day = 0;
    lf->hour = NULL;

    return;
}

/* Free the loginfo structure */
void Free_Eventinfo(Eventinfo *lf)
{
    if(!lf)
    {
        merror("%s: Trying to free NULL event. Inconsistent..",ARGV0);
        return;
    }
    
    if(lf->log)
        free(lf->log);
    if(lf->group)
        free(lf->group);
    if(lf->location)
        free(lf->location);
    if(lf->hostname)
        free(lf->hostname);    
    if(lf->info)
        free(lf->info);    

    if(lf->srcip)
        free(lf->srcip);
    if(lf->dstip)
        free(lf->dstip);
    if(lf->user)
        free(lf->user);
    if(lf->dstuser)
        free(lf->dstuser);    
    if(lf->id)
        free(lf->id);
    if(lf->command)
        free(lf->command);
        
    if(lf->mon)
        free(lf->mon);
    if(lf->hour)
        free(lf->hour);            

    /* We dont need to free:
     * log_tag
     * fts
     * comment
     */
    
    free(lf);
    lf = NULL; 
    
    return;
}	

/* EOF */
