/*   $OSSEC, logcollector.c, v0.1, 2005/04/04, Daniel B. Cid$   */

/* Copyright (C) 2003,2004,2005 Daniel B. Cid <dcid@ossec.net>
 * All right reserved.
 *
 * This program is a free software; you can redistribute it
 * and/or modify it under the terms of the GNU General Public
 * License (version 2) as published by the FSF - Free Software
 * Foundation
 */


/* Logcolletor daemon.
 * Monitor some files and forward the output to our analysis system.
 */

#include <stdio.h>
#include <unistd.h>
#include <stdlib.h>
#include <string.h>

#ifndef ARGV0
   #define ARGV0="ossec-logcollector"
#endif
      
#include "headers/defs.h"
#include "headers/mq_op.h"
#include "headers/sig_op.h"
#include "headers/file_op.h"
#include "headers/debug_op.h"
#include "headers/help.h"
#include "headers/privsep_op.h"
#include "headers/os_err.h"
#include "os_regex/os_regex.h"

#include "error_messages/error_messages.h"

#include "logcollector.h"

short int dbg_flag=0;
short int chroot_flag=0;


/* Necessary internal global var */
short int _checksnortfull=0;    /* For when reading the snort full */
collector logcollect;           /* Log information */

/* Internal functions */
int readfile();
void run();

/* main: v0.3: 2005/04/04 */
int main(int argc, char **argv)
{
    int i=0, c;
    char *cfg=DEFAULTCPATH;
    char *dir=DEFAULTDIR;
    logreader logr;

    while((c = getopt(argc, argv, "dhD:c:")) != -1)
    {
        switch(c)
        {
            case 'h':
                help();
                break;
            case 'd':
                dbg_flag++;
                break;
            case 'D':
                if(!optarg)
                    ErrorExit("%s: -D needs an argument",ARGV0);
                dir = optarg;
                break;
            case 'c':
                if(!optarg)
                    ErrorExit("%s: -c needs an argument",ARGV0);
                cfg = optarg;
                break;
            default:
                help();
                break;   
        }

    }

    debug1(STARTED_MSG,ARGV0);

    if(File_DateofChange(cfg) < 0)
        ErrorExit("%s: Configuration file: %s not found",ARGV0,cfg);

    /* Starting the queue. */
    if((logcollect.queue = StartMQ(DEFAULTQPATH,WRITE)) < 0)
    {   
        merror(QUEUE_ERROR,ARGV0,DEFAULTQPATH);
        sleep(10);
        if((logcollect.queue = StartMQ(DEFAULTQPATH,WRITE)) < 0)
        {
            merror(QUEUE_ERROR,ARGV0,DEFAULTQPATH);
            sleep(60);
            if((logcollect.queue = StartMQ(DEFAULTQPATH,WRITE)) < 0)
                ErrorExit(QUEUE_FATAL,ARGV0,DEFAULTQPATH);
        }
    }

    /* Starting signal handler */
    StartSIG(ARGV0);	

    if((c=FilesConf(cfg,&logr)) == OS_NOTFOUND)
    {
        verbose("%s: No file configured to monitor. Exiting.",ARGV0);
        exit(0);
    }
    else if(c < 0)
        ErrorExit(CONFIG_ERROR,ARGV0,cfg);

    #ifdef DEBUG
        verbose("%s: Going to read files and fork..",ARGV0);
    #endif
    
    /* Looping and creating a new process for each file */
    for(i=0;i<c;i++)
    {
        int pid=0;
        pid = fork();
        if(pid < 0)
            ErrorExit(FORK_ERROR,ARGV0);
        else if(pid == 0)
        {
            /* Creating PID file */
            
            #ifdef DEBUG
            verbose("%s: New process (%d) for file %s\n",ARGV0,getpid(),
                                                        logr.file[i]);
            #endif
            
            if(CreatePID(ARGV0, getpid()) < 0)
                ErrorExit(PID_ERROR,ARGV0);
            
            logcollect.file = logr.file[i];
            
            logcollect.group= logr.group[i];
            
            run();
            return(0);
        }
        else
            continue;
    }

    /* Exiting from the main process */
    exit(0);
}


/* This process monitor the file and forward their content to analysisd */
void run()
{
    int ign = 0;
    int r = 0,tmpmtime = 0;

    /* Initializing the variables, testing the file
     * and setting the file pointer to the right location
     */

    ign = 0;
    logcollect.fp = fopen(logcollect.file, "r");
    if(!logcollect.fp)
    {
        ign = 1;
        merror(FOPEN_ERROR,ARGV0,logcollect.file);
    }

    else if(fseek(logcollect.fp, 0, SEEK_END) < 0)
        ign = 1;	
    else
    {
        logcollect.mtime=File_DateofChange(logcollect.file);
        if(logcollect.mtime < 0)
            ign = 1;
    }

    /* Setting log type */
	logcollect.type = LOCALFILE_MQ;	    /* Default to localfile */
   
	if(OS_Match("snort-full", logcollect.group))
    {
        logcollect.type = SNORT_MQ;
        _checksnortfull = 1;
    }

	else if(OS_Match("snort-fast", logcollect.group))
		logcollect.type = SNORT_MQ;

	else if(OS_Match("apache-err", logcollect.group))
		logcollect.type = APACHERR_MQ;
  
         
     
    /* Daemon */	
    while(1)
    {
        usleep(1000); /* Small sleep time 1000 usec */
        tmpmtime = File_DateofChange(logcollect.file);

        if(tmpmtime < 0)
        {
            if(ign == 0)
            {
                merror("%s: Problems with the file %s",ARGV0, logcollect.file);
                ign = 1;
            }
        }
        
        else if(logcollect.mtime == tmpmtime)
            continue;
            
        else
        {

            logcollect.mtime = tmpmtime;
            
            #ifdef DEBUG
            verbose("%s: DEBUG: File date changed (%s)",ARGV0,logcollect.file);
            #endif

            if(ign > 0 && (!logcollect.fp))
            {
                logcollect.fp = fopen(logcollect.file,"r");
                if(!logcollect.fp)
                {
                    ign=1;
                    continue;
                }
                
                fseek(logcollect.fp,0,SEEK_END);
            }
            
            r = readfile();
            
            if(r == 0)
            {
                fseek(logcollect.fp, 0, SEEK_CUR); /* bug with EOF */
                ign = 0;
                logcollect.mtime = File_DateofChange(logcollect.file);
            }
                
            else
            {
                
                #ifdef DEBUG
                verbose("%s: File error (%s)",ARGV0,logcollect.file);
                #endif
                
                ign++;
                fclose(logcollect.fp);
                
                logcollect.fp=fopen(logcollect.file,"r");
                
                if(!logcollect.fp)
                    ign++;
                else if(ign > 1) /* File removed ?*/
                    fseek(logcollect.fp,0,SEEK_END);
                else
                    fseek(logcollect.fp,r,SEEK_SET);
            }
        }	
    }
}


/* This function read the file itself */
int readfile()
{

    #ifdef DEBUG
    verbose("%s: DEBUG: On readfile...",ARGV0);
    #endif
    
    /* If it is snort full */
    if(_checksnortfull == 1)
    {
        return(read_snortfull());
    }

    else
    {
        return(read_syslog());
    }
    
}
